@@ -1,4 +1,4 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import os
 from itertools import chain
@@ -15,7 +15,8 @@ import decimal
 from decimal import Decimal
 import math
 from tryton.common import RPCExecute, RPCException
-import tryton.rpc as rpc
+from tryton.pyson import PYSONDecoder
+from tryton.config import CONFIG
 
 
 class Field(object):
@@ -34,6 +35,7 @@ class Field(object):
     def __init__(self, attrs):
         self.attrs = attrs
         self.name = attrs['name']
+        self.views = set()
 
     def sig_changed(self, record):
         record.on_change([self.name])
@@ -59,13 +61,23 @@ class Field(object):
     def validation_domains(self, record, pre_validate=None):
         return concat(*self.domains_get(record, pre_validate))
 
-    def get_context(self, record):
-        context = record.get_context().copy()
-        if record.parent:
-            context.update(record.parent.get_context())
+    def get_context(self, record, record_context=None):
+        if record_context is not None:
+            context = record_context.copy()
+        else:
+            context = record.get_context()
         context.update(record.expr_eval(self.attrs.get('context', {})))
         return context
 
+    def get_search_context(self, record):
+        context = self.get_context(record)
+        context.update(record.expr_eval(self.attrs.get('search_context', {})))
+        return context
+
+    def get_search_order(self, record):
+        order = record.expr_eval(self.attrs.get('search_order', None))
+        return order
+
     def _is_empty(self, record):
         return not self.get_eval(record)
 
@@ -231,7 +243,7 @@ class DateTimeField(Field):
 
     def date_format(self, record):
         context = self.get_context(record)
-        return context.get('date_format', '%x')
+        return common.date_format(context.get('date_format'))
 
     def time_format(self, record):
         return record.expr_eval(self.attrs['format'])
@@ -250,7 +262,7 @@ class DateField(Field):
 
     def date_format(self, record):
         context = self.get_context(record)
-        return context.get('date_format', '%x')
+        return common.date_format(context.get('date_format'))
 
 
 class TimeField(Field):
@@ -277,19 +289,18 @@ class TimeDeltaField(Field):
     def _is_empty(self, record):
         return self.get(record) is None
 
-    def converter(self, record):
-        # TODO allow local context converter
-        return rpc.CONTEXT.get(self.attrs.get('converter'))
+    def converter(self, group):
+        return group.context.get(self.attrs.get('converter'))
 
     def set_client(self, record, value, force_change=False):
-        if isinstance(value, basestring):
-            value = common.timedelta.parse(value, self.converter(record))
+        if isinstance(value, str):
+            value = common.timedelta.parse(value, self.converter(record.group))
         super(TimeDeltaField, self).set_client(
             record, value, force_change=force_change)
 
     def get_client(self, record):
         value = super(TimeDeltaField, self).get_client(record)
-        return common.timedelta.format(value, self.converter(record))
+        return common.timedelta.format(value, self.converter(record.group))
 
 
 class FloatField(Field):
@@ -315,10 +326,14 @@ class FloatField(Field):
             return self._default
 
     def set_client(self, record, value, force_change=False, factor=1):
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             value = self.convert(value)
         if value is not None:
-            value /= factor
+            # Keep the same type
+            if isinstance(value, int):
+                value //= factor
+            else:
+                value /= factor
         super(FloatField, self).set_client(record, value,
             force_change=force_change)
 
@@ -327,7 +342,7 @@ class FloatField(Field):
         if value is not None:
             digits = self.digits(record, factor=factor)
             if digits:
-                p = digits[1]
+                p = int(digits[1])
             else:
                 d = value * factor
                 if not isinstance(d, Decimal):
@@ -398,38 +413,39 @@ class M2OField(Field):
         return self.get(record) is None
 
     def get_client(self, record):
-        rec_name = record.value.get(self.name + '.rec_name')
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name')
         if rec_name is None:
             self.set(record, self.get(record))
-            rec_name = record.value.get(self.name + '.rec_name') or ''
-        return rec_name
+            rec_name = record.value.get(self.name + '.', {}).get('rec_name')
+        return rec_name or ''
 
     def set_client(self, record, value, force_change=False):
         if isinstance(value, (tuple, list)):
             value, rec_name = value
         else:
             if value == self.get(record):
-                rec_name = record.value.get(self.name + '.rec_name', '')
+                rec_name = record.value.get(
+                    self.name + '.', {}).get('rec_name', '')
             else:
                 rec_name = ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         super(M2OField, self).set_client(record, value,
             force_change=force_change)
 
     def set(self, record, value):
-        rec_name = record.value.get(self.name + '.rec_name') or ''
-        if not rec_name and value >= 0:
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
+        if not rec_name and value is not None and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
                     [value], ['rec_name'])
             except RPCException:
                 return
             rec_name = result['rec_name'] or ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         record.value[self.name] = value
 
-    def get_context(self, record):
-        context = super(M2OField, self).get_context(record)
+    def get_context(self, record, record_context=None):
+        context = super(M2OField, self).get_context(record, record_context)
         if self.attrs.get('datetime_field'):
             context['_datetime'] = record.get_eval(
                 )[self.attrs.get('datetime_field')]
@@ -491,7 +507,7 @@ class O2MField(Field):
     def _set_default_value(self, record, fields=None):
         if record.value.get(self.name) is not None:
             return
-        from group import Group
+        from .group import Group
         parent_name = self.attrs.get('relation_field', '')
         fields = fields or {}
         context = record.expr_eval(self.attrs.get('context', {}))
@@ -588,7 +604,7 @@ class O2MField(Field):
         group = record.value[self.name]
         if value is None:
             value = []
-        if not value or isinstance(value[0], (int, long)):
+        if not value or isinstance(value[0], int):
             mode = 'list ids'
         else:
             mode = 'list values'
@@ -635,7 +651,7 @@ class O2MField(Field):
             fields = record.group.fields
         if fields:
             fields = dict((fname, field.attrs)
-                for fname, field in fields.iteritems())
+                for fname, field in fields.items())
 
         record.value[self.name] = None
         self._set_default_value(record, fields=fields)
@@ -650,7 +666,7 @@ class O2MField(Field):
         if value is None:
             value = []
         # domain inversion could try to set id as value
-        if isinstance(value, (int, long)):
+        if isinstance(value, int):
             value = [value]
 
         previous_ids = self.get_eval(record)
@@ -685,7 +701,7 @@ class O2MField(Field):
             values = chain(value.get('update', []),
                 (d for _, d in value.get('add', [])))
             field_names = set(f for v in values
-                for f in v if f not in fields and f != 'id')
+                for f in v if f not in fields and f != 'id' and '.' not in f)
             if field_names:
                 try:
                     fields = RPCExecute('model', self.attrs['relation'],
@@ -775,14 +791,15 @@ class ReferenceField(Field):
 
     def _is_empty(self, record):
         result = super(ReferenceField, self)._is_empty(record)
-        if not result and record.value[self.name][1] < 0:
+        if not result and (record.value[self.name] is None
+                or record.value[self.name][1] < 0):
             result = True
         return result
 
     def get_client(self, record):
         if record.value.get(self.name):
             model, _ = record.value[self.name]
-            name = record.value.get(self.name + '.rec_name') or ''
+            name = record.value.get(self.name + '.', {}).get('rec_name') or ''
             return model, name
         else:
             return None
@@ -790,13 +807,14 @@ class ReferenceField(Field):
     def get(self, record):
         if (record.value.get(self.name)
                 and record.value[self.name][0]
+                and record.value[self.name][1] is not None
                 and record.value[self.name][1] >= -1):
             return ','.join(map(str, record.value[self.name]))
         return None
 
     def set_client(self, record, value, force_change=False):
         if value:
-            if isinstance(value, basestring):
+            if isinstance(value, str):
                 value = value.split(',')
             ref_model, ref_id = value
             if isinstance(ref_id, (tuple, list)):
@@ -808,10 +826,11 @@ class ReferenceField(Field):
                     except ValueError:
                         pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
-                    rec_name = record.value.get(self.name + '.rec_name', '')
+                    rec_name = record.value.get(
+                        self.name + '.', {}).get('rec_name', '')
                 else:
                     rec_name = ''
-            record.value[self.name + '.rec_name'] = rec_name
+            record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
             value = (ref_model, ref_id)
         super(ReferenceField, self).set_client(record, value,
             force_change=force_change)
@@ -820,7 +839,7 @@ class ReferenceField(Field):
         if not value:
             record.value[self.name] = self._default
             return
-        if isinstance(value, basestring):
+        if isinstance(value, str):
             ref_model, ref_id = value.split(',')
             if not ref_id:
                 ref_id = None
@@ -831,8 +850,8 @@ class ReferenceField(Field):
                     pass
         else:
             ref_model, ref_id = value
-        rec_name = record.value.get(self.name + '.rec_name') or ''
-        if ref_model and ref_id >= 0:
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
+        if ref_model and ref_id is not None and ref_id >= 0:
             if not rec_name and ref_id >= 0:
                 try:
                     result, = RPCExecute('model', ref_model, 'read', [ref_id],
@@ -845,10 +864,11 @@ class ReferenceField(Field):
         else:
             rec_name = str(ref_id)
         record.value[self.name] = ref_model, ref_id
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
 
-    def get_context(self, record):
-        context = super(ReferenceField, self).get_context(record)
+    def get_context(self, record, record_context=None):
+        context = super(ReferenceField, self).get_context(
+            record, record_context)
         if self.attrs.get('datetime_field'):
             context['_datetime'] = record.get_eval(
                 )[self.attrs.get('datetime_field')]
@@ -885,18 +905,23 @@ class _FileCache(object):
     def __init__(self, path):
         self.path = path
 
+    def __del__(self):
+        try:
+            os.remove(self.path)
+        except IOError:
+            pass
+
 
 class BinaryField(Field):
 
     _default = None
-    cast = bytearray if bytes == str else bytes
 
     def get(self, record):
         result = record.value.get(self.name, self._default)
         if isinstance(result, _FileCache):
             try:
                 with open(result.path, 'rb') as fp:
-                    result = self.cast(fp.read())
+                    result = fp.read()
             except IOError:
                 result = self.get_data(record)
         return result
@@ -906,8 +931,11 @@ class BinaryField(Field):
 
     def set_client(self, record, value, force_change=False):
         _, filename = tempfile.mkstemp(prefix='tryton_')
+        data = value or b''
+        if isinstance(data, str):
+            data = data.encode('utf-8')
         with open(filename, 'wb') as fp:
-            fp.write(value or '')
+            fp.write(data)
         self.set(record, _FileCache(filename))
         record.modified_fields.setdefault(self.name)
         record.signal('record-modified')
@@ -919,24 +947,27 @@ class BinaryField(Field):
         result = record.value.get(self.name) or 0
         if isinstance(result, _FileCache):
             result = os.stat(result.path).st_size
-        elif isinstance(result, (basestring, bytes, bytearray)):
+        elif isinstance(result, (str, bytes)):
             result = len(result)
         return result
 
     def get_data(self, record):
         if not isinstance(record.value.get(self.name),
-                (basestring, bytes, bytearray, _FileCache)):
+                (str, bytes, _FileCache)):
             if record.id < 0:
-                return ''
+                return b''
             context = record.get_context()
             try:
                 values, = RPCExecute('model', record.model_name, 'read',
                     [record.id], [self.name], context=context)
             except RPCException:
-                return ''
+                return b''
             _, filename = tempfile.mkstemp(prefix='tryton_')
+            data = values[self.name] or b''
+            if isinstance(data, str):
+                data = data.encode('utf-8')
             with open(filename, 'wb') as fp:
-                fp.write(values[self.name] or '')
+                fp.write(data)
             self.set(record, _FileCache(filename))
         return self.get(record)
 
@@ -945,6 +976,10 @@ class DictField(Field):
 
     _default = {}
 
+    def __init__(self, attrs):
+        super(DictField, self).__init__(attrs)
+        self.keys = {}
+
     def get(self, record):
         return super(DictField, self).get(record) or self._default
 
@@ -961,11 +996,76 @@ class DictField(Field):
 
     def date_format(self, record):
         context = self.get_context(record)
-        return context.get('date_format', '%x')
+        return common.date_format(context.get('date_format'))
 
     def time_format(self, record):
         return '%X'
 
+    def add_keys(self, keys, record):
+        schema_model = self.attrs['schema_model']
+        context = self.get_context(record)
+        domain = self.domain_get(record)
+        batchlen = min(10, CONFIG['client.limit'])
+        for i in range(0, len(keys), batchlen):
+            sub_keys = keys[i:i + batchlen]
+            try:
+                key_ids = RPCExecute('model', schema_model, 'search',
+                    [('name', 'in', sub_keys), domain], 0,
+                    CONFIG['client.limit'], None, context=context)
+            except RPCException:
+                key_ids = []
+            if not key_ids:
+                continue
+            try:
+                values = RPCExecute('model', schema_model,
+                    'get_keys', key_ids, context=context)
+            except RPCException:
+                values = []
+            if not values:
+                continue
+            self.keys.update({k['name']: k for k in values})
+
+    def add_new_keys(self, key_ids, record):
+        schema_model = self.attrs['schema_model']
+        context = self.get_context(record)
+        try:
+            new_fields = RPCExecute('model', schema_model,
+                'get_keys', key_ids, context=context)
+        except RPCException:
+            new_fields = []
+
+        new_keys = []
+        for new_field in new_fields:
+            name = new_field['name']
+            new_keys.append(name)
+            self.keys[name] = new_field
+
+        return new_keys
+
+    def validate(self, record, softvalidation=False, pre_validate=None):
+        valid = super(DictField, self).validate(
+            record, softvalidation, pre_validate)
+
+        if self.attrs.get('readonly'):
+            return valid
+
+        decoder = PYSONDecoder()
+        field_value = self.get_eval(record)
+        domain = []
+        for key in field_value:
+            if key not in self.keys:
+                continue
+            key_domain = self.keys[key].get('domain')
+            if key_domain:
+                domain.append(decoder.decode(key_domain))
+
+        valid_value = eval_domain(domain, field_value)
+        if not valid_value:
+            self.get_state_attrs(record)['invalid'] = 'domain'
+
+        return valid and valid_value
+
+
 TYPES = {
     'char': CharField,
     'integer': IntegerField,
@@ -981,6 +1081,7 @@ TYPES = {
     'datetime': DateTimeField,
     'date': DateField,
     'time': TimeField,
+    'timestamp': DateTimeField,
     'timedelta': TimeDeltaField,
     'one2one': O2OField,
     'binary': BinaryField,
