@@ -1,11 +1,9 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
 import tryton.common as common
 from tryton.pyson import PYSONDecoder
-import field as fields
-from functools import reduce
+from . import field as fields
 from tryton.common import RPCExecute, RPCException
 from tryton.config import CONFIG
 
@@ -30,8 +28,7 @@ class Record(SignalEvent):
         self.state_attrs = {}
         self.modified_fields = {}
         self._timestamp = None
-        self.attachment_count = -1
-        self.unread_note = -1
+        self.resources = None
         self.button_clicks = {}
         self.next = {}  # Used in Group list
         self.value = {}
@@ -39,32 +36,42 @@ class Record(SignalEvent):
         self.exception = False
         self.destroyed = False
 
+        # GNU Health client 3.6
+        self.attachment_count = -1
+        self.unread_note = -1
+
+
     def __getitem__(self, name):
         if name not in self._loaded and self.id >= 0:
             id2record = {
                 self.id: self,
                 }
             if name == '*':
-                loading = reduce(
-                        lambda x, y: 'eager' if x == y == 'eager' else 'lazy',
-                        (field.attrs.get('loading', 'eager')
-                            for field in self.group.fields.itervalues()),
-                        'eager')
+                loading = 'eager'
+                views = set()
+                for field in self.group.fields.values():
+                    if field.attrs.get('loading', 'eager') == 'lazy':
+                        loading = 'lazy'
+                    views |= field.views
                 # Set a valid name for next loaded check
-                for fname, field in self.group.fields.iteritems():
+                for fname, field in self.group.fields.items():
                     if field.attrs.get('loading', 'eager') == loading:
                         name = fname
                         break
             else:
                 loading = self.group.fields[name].attrs.get('loading', 'eager')
+                views = self.group.fields[name].views
 
             if loading == 'eager':
-                fnames = [fname
-                    for fname, field in self.group.fields.iteritems()
-                    if field.attrs.get('loading', 'eager') == 'eager']
+                fields = ((fname, field)
+                    for fname, field in self.group.fields.items()
+                    if field.attrs.get('loading', 'eager') == 'eager')
             else:
-                fnames = self.group.fields.keys()
-            fnames = [fname for fname in fnames if fname not in self._loaded]
+                fields = self.group.fields.items()
+
+            fnames = [fname for fname, field in fields
+                if fname not in self._loaded
+                and (not views or (views & field.views))]
             fnames.extend(('%s.rec_name' % fname for fname in fnames[:]
                     if self.group.fields[fname].attrs['type']
                     in ('many2one', 'one2one', 'reference')))
@@ -110,12 +117,12 @@ class Record(SignalEvent):
 
             ctx = record_context.copy()
             ctx.update(dict(('%s.%s' % (self.model_name, fname), 'size')
-                    for fname, field in self.group.fields.iteritems()
+                    for fname, field in self.group.fields.items()
                     if field.attrs['type'] == 'binary' and fname in fnames))
             exception = False
             try:
                 values = RPCExecute('model', self.model_name, 'read',
-                    id2record.keys(), fnames, context=ctx)
+                    list(id2record.keys()), fnames, context=ctx)
             except RPCException:
                 values = [{'id': x} for x in id2record]
                 default_values = dict((f, None) for f in fnames)
@@ -123,7 +130,7 @@ class Record(SignalEvent):
                     value.update(default_values)
                 self.exception = exception = True
             id2value = dict((value['id'], value) for value in values)
-            for id, record in id2record.iteritems():
+            for id, record in id2record.items():
                 if not record.exception:
                     record.exception = exception
                 value = id2value.get(id)
@@ -131,7 +138,40 @@ class Record(SignalEvent):
                     for key in record.modified_fields:
                         value.pop(key, None)
                     record.set(value, signal=False)
-        return self.group.fields.get(name, False)
+        return self.group.fields[name]
+
+    #GNU Health 3.6 client notes and attachment counts
+    def get_attachment_count(self, reload=False):
+        if self.id < 0:
+            return 0
+        if self.attachment_count < 0 or reload:
+            try:
+                self.attachment_count = RPCExecute('model', 'ir.attachment',
+                    'search_count', [
+                        ('resource', '=',
+                            '%s,%s' % (self.model_name, self.id)),
+                        ], context=self.get_context())
+            except RPCException:
+                return 0
+        return self.attachment_count
+
+    def get_unread_note(self, reload=False):
+        if self.id < 0:
+            return 0
+        if self.unread_note < 0 or reload:
+            try:
+                self.unread_note = RPCExecute('model', 'ir.note',
+                    'search_count', [
+                        ('resource', '=',
+                            '%s,%s' % (self.model_name, self.id)),
+                        ('unread', '=', True),
+                        ], context=self.get_context())
+            except RPCException:
+                return 0
+        return self.unread_note
+
+
+
 
     def __repr__(self):
         return '<Record %s@%s at %s>' % (self.id, self.model_name, id(self))
@@ -250,9 +290,10 @@ class Record(SignalEvent):
 
     def get(self):
         value = {}
-        for name, field in self.group.fields.iteritems():
+        for name, field in self.group.fields.items():
             if (field.attrs.get('readonly')
-                    and not isinstance(field, fields.O2MField)):
+                    and not (isinstance(field, fields.O2MField)
+                        and not isinstance(field, fields.M2MField))):
                 continue
             if field.name not in self.modified_fields and self.id >= 0:
                 continue
@@ -264,7 +305,7 @@ class Record(SignalEvent):
 
     def get_eval(self):
         value = {}
-        for name, field in self.group.fields.iteritems():
+        for name, field in self.group.fields.items():
             if name not in self._loaded and self.id >= 0:
                 continue
             value[name] = field.get_eval(self)
@@ -273,7 +314,7 @@ class Record(SignalEvent):
 
     def get_on_change_value(self, skip=None):
         value = {}
-        for name, field in self.group.fields.iteritems():
+        for name, field in self.group.fields.items():
             if skip and name in skip:
                 continue
             if (self.id >= 0
@@ -292,7 +333,7 @@ class Record(SignalEvent):
 
     def get_timestamp(self):
         result = {self.model_name + ',' + str(self.id): self._timestamp}
-        for name, field in self.group.fields.iteritems():
+        for name, field in self.group.fields.items():
             if name in self._loaded:
                 result.update(field.get_timestamp(self))
         return result
@@ -341,39 +382,13 @@ class Record(SignalEvent):
             self.parent.save(force_reload=force_reload)
         return self.id
 
-    @staticmethod
-    def delete(records):
-        if not records:
-            return
-        record = records[0]
-        root_group = record.group.root_group
-        assert all(r.model_name == record.model_name for r in records)
-        assert all(r.group.root_group == root_group for r in records)
-        records = [r for r in records if r.id >= 0]
-        ctx = {}
-        ctx['_timestamp'] = {}
-        for rec in records:
-            ctx['_timestamp'].update(rec.get_timestamp())
-        record_ids = set(r.id for r in records)
-        reload_ids = set(root_group.on_write_ids(list(record_ids)))
-        reload_ids -= record_ids
-        reload_ids = list(reload_ids)
-        try:
-            RPCExecute('model', record.model_name, 'delete', list(record_ids),
-                context=ctx)
-        except RPCException:
-            return False
-        if reload_ids:
-            root_group.reload(reload_ids)
-        return True
-
     def default_get(self, rec_name=None):
         if len(self.group.fields):
             context = self.get_context()
             context.setdefault('default_rec_name', rec_name)
             try:
                 vals = RPCExecute('model', self.model_name, 'default_get',
-                    self.group.fields.keys(), context=context)
+                    list(self.group.fields.keys()), context=context)
             except RPCException:
                 return
             if (self.parent
@@ -401,7 +416,7 @@ class Record(SignalEvent):
         elif fields is None:
             self._check_load()
         res = True
-        for field_name, field in self.group.fields.iteritems():
+        for field_name, field in self.group.fields.items():
             if fields is not None and field_name not in fields:
                 continue
             if field.attrs.get('readonly'):
@@ -414,7 +429,7 @@ class Record(SignalEvent):
 
     def _get_invalid_fields(self):
         fields = {}
-        for fname, field in self.group.fields.iteritems():
+        for fname, field in self.group.fields.items():
             invalid = field.get_state_attrs(self).get('invalid')
             if invalid:
                 fields[fname] = invalid
@@ -427,18 +442,15 @@ class Record(SignalEvent):
 
     def set_default(self, val, signal=True, validate=True):
         fieldnames = []
-        for fieldname, value in val.items():
+        for fieldname, value in list(val.items()):
             if fieldname not in self.group.fields:
                 continue
             if fieldname == self.group.exclude_field:
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in val:
-                    self.value[field_rec_name] = val[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                related = fieldname + '.'
+                self.value[related] = val.get(related) or {}
             self.group.fields[fieldname].set_default(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
@@ -452,7 +464,7 @@ class Record(SignalEvent):
     def set(self, val, signal=True, validate=True):
         later = {}
         fieldnames = []
-        for fieldname, value in val.iteritems():
+        for fieldname, value in val.items():
             if fieldname == '_timestamp':
                 # Always keep the older timestamp
                 if not self._timestamp:
@@ -467,15 +479,12 @@ class Record(SignalEvent):
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in val:
-                    self.value[field_rec_name] = val[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                related = fieldname + '.'
+                self.value[related] = val.get(related) or {}
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
             fieldnames.append(fieldname)
-        for fieldname, value in later.iteritems():
+        for fieldname, value in later.items():
             self.group.fields[fieldname].set(self, value)
             self._loaded.add(fieldname)
         if validate:
@@ -484,16 +493,13 @@ class Record(SignalEvent):
             self.signal('record-changed')
 
     def set_on_change(self, values):
-        for fieldname, value in values.items():
+        for fieldname, value in list(values.items()):
             if fieldname not in self.group.fields:
                 continue
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in values:
-                    self.value[field_rec_name] = values[field_rec_name]
-                elif field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                related = fieldname + '.'
+                self.value[related] = values.get(related) or {}
             self.group.fields[fieldname].set_on_change(self, value)
 
     def reload(self, fields=None):
@@ -506,8 +512,18 @@ class Record(SignalEvent):
                 self[field]
         self.validate(fields or [])
 
+    def reset(self, value):
+        self.cancel()
+        self.set(value, signal=False)
+
+        if self.parent:
+            self.parent.on_change([self.group.child_name])
+            self.parent.on_change_with([self.group.child_name])
+
+        self.signal('record-changed')
+
     def expr_eval(self, expr):
-        if not isinstance(expr, basestring):
+        if not isinstance(expr, str):
             return expr
         if not expr:
             return
@@ -515,13 +531,10 @@ class Record(SignalEvent):
             return []
         elif expr == '{}':
             return {}
-        ctx = rpc.CONTEXT.copy()
-        ctx['context'] = ctx.copy()
-        ctx['context'].update(self.get_context())
-        ctx.update(self.get_eval())
+        ctx = self.get_eval()
+        ctx['context'] = self.get_context()
         ctx['active_model'] = self.model_name
         ctx['active_id'] = self.id
-        ctx['_user'] = rpc._USER
         if self.parent and self.parent_name:
             ctx['_parent_' + self.parent_name] = \
                 common.EvalEnvironment(self.parent)
@@ -539,7 +552,6 @@ class Record(SignalEvent):
                 scope = scope[i]
             else:
                 res[arg] = scope
-        res['id'] = self.id
         return res
 
     def on_change(self, fieldnames):
@@ -552,8 +564,16 @@ class Record(SignalEvent):
 
         if values:
             try:
-                changes = RPCExecute('model', self.model_name, 'on_change',
-                    values, fieldnames, context=self.get_context())
+                if len(fieldnames) == 1:
+                    fieldname, = fieldnames
+                    changes = []
+                    changes.append(RPCExecute(
+                            'model', self.model_name, 'on_change_' + fieldname,
+                            values, context=self.get_context()))
+                else:
+                    changes = RPCExecute(
+                        'model', self.model_name, 'on_change',
+                        values, fieldnames, context=self.get_context())
             except RPCException:
                 return
             for change in changes:
@@ -578,13 +598,20 @@ class Record(SignalEvent):
             values.update(self._get_on_change_args(on_change_with))
             if isinstance(self.group.fields[fieldname], (fields.M2OField,
                         fields.ReferenceField)):
-                field_rec_name = fieldname + '.rec_name'
-                if field_rec_name in self.value:
-                    del self.value[field_rec_name]
+                self.value.pop(fieldname + '.', None)
         if fieldnames:
             try:
-                result = RPCExecute('model', self.model_name, 'on_change_with',
-                    values, list(fieldnames), context=self.get_context())
+                if len(fieldnames) == 1:
+                    fieldname, = fieldnames
+                    result = {}
+                    result[fieldname] = RPCExecute(
+                        'model', self.model_name,
+                        'on_change_with_' + fieldname,
+                        values, context=self.get_context())
+                else:
+                    result = RPCExecute(
+                        'model', self.model_name, 'on_change_with',
+                        values, list(fieldnames), context=self.get_context())
             except RPCException:
                 return
             self.set_on_change(result)
@@ -601,7 +628,7 @@ class Record(SignalEvent):
             self.group.fields[fieldname].set_on_change(self, result)
 
     def autocomplete_with(self, field_name):
-        for fieldname, fieldinfo in self.group.fields.iteritems():
+        for fieldname, fieldinfo in self.group.fields.items():
             autocomplete = fieldinfo.attrs.get('autocomplete', [])
             if field_name not in autocomplete:
                 continue
@@ -621,7 +648,7 @@ class Record(SignalEvent):
 
     def set_field_context(self):
         from .group import Group
-        for name, field in self.group.fields.iteritems():
+        for name, field in self.group.fields.items():
             value = self.value.get(name)
             if not isinstance(value, Group):
                 continue
@@ -629,34 +656,15 @@ class Record(SignalEvent):
             if context:
                 value.context = self.expr_eval(context)
 
-    def get_attachment_count(self, reload=False):
-        if self.id < 0:
-            return 0
-        if self.attachment_count < 0 or reload:
+    def get_resources(self, reload=False):
+        if self.id >= 0 and (not self.resources or reload):
             try:
-                self.attachment_count = RPCExecute('model', 'ir.attachment',
-                    'search_count', [
-                        ('resource', '=',
-                            '%s,%s' % (self.model_name, self.id)),
-                        ], context=self.get_context())
+                self.resources = RPCExecute(
+                    'model', self.model_name, 'resources', self.id,
+                    context=self.get_context())
             except RPCException:
-                return 0
-        return self.attachment_count
-
-    def get_unread_note(self, reload=False):
-        if self.id < 0:
-            return 0
-        if self.unread_note < 0 or reload:
-            try:
-                self.unread_note = RPCExecute('model', 'ir.note',
-                    'search_count', [
-                        ('resource', '=',
-                            '%s,%s' % (self.model_name, self.id)),
-                        ('unread', '=', True),
-                        ], context=self.get_context())
-            except RPCException:
-                return 0
-        return self.unread_note
+                pass
+        return self.resources
 
     def get_button_clicks(self, name):
         if self.id < 0:
@@ -673,7 +681,7 @@ class Record(SignalEvent):
         return clicks
 
     def destroy(self):
-        for v in self.value.itervalues():
+        for v in self.value.values():
             if hasattr(v, 'destroy'):
                 v.destroy()
         super(Record, self).destroy()
