@@ -1,20 +1,19 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 "Screen"
 import copy
 import functools
-import gobject
 import datetime
 import calendar
 import json
 import collections
-import urllib
-import urlparse
+import urllib.parse
 import xml.dom.minidom
 import gettext
 import logging
+from operator import itemgetter
 
-import gtk
+from gi.repository import GLib, Gtk
 
 from tryton.gui.window.view_form.model.group import Group
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
@@ -27,7 +26,7 @@ from tryton.common import RPCExecute, RPCException, MODELACCESS, \
     node_attributes, sur, RPCContextReload, warning
 from tryton.action import Action
 from tryton.pyson import PYSONDecoder
-import tryton.rpc as rpc
+from tryton.rpc import clear_cache
 
 _ = gettext.gettext
 logger = logging.getLogger(__name__)
@@ -94,19 +93,19 @@ class Screen(SignalEvent):
                         yield widget
 
             for widget in reversed(list(walk_descendants(context_widget))):
-                if isinstance(widget, gtk.Entry):
+                if isinstance(widget, Gtk.Entry):
                     widget.connect_after(
                         'activate', self.screen_container.activate)
-                elif isinstance(widget, gtk.CheckButton):
+                elif isinstance(widget, Gtk.CheckButton):
                     widget.connect_after(
                         'toggled', self.screen_container.activate)
 
             def remove_bin(widget):
-                assert isinstance(widget, (gtk.ScrolledWindow, gtk.Viewport))
+                assert isinstance(widget, (Gtk.ScrolledWindow, Gtk.Viewport))
                 parent = widget.get_parent()
                 parent.remove(widget)
                 child = widget.get_child()
-                while isinstance(child, (gtk.ScrolledWindow, gtk.Viewport)):
+                while isinstance(child, (Gtk.ScrolledWindow, Gtk.Viewport)):
                     child = child.get_child()
                 child.get_parent().remove(child)
                 parent.add(child)
@@ -120,7 +119,7 @@ class Screen(SignalEvent):
                 remove_bin(widget)
 
             self.screen_container.filter_vbox.pack_start(
-                context_widget, expand=False, fill=True)
+                context_widget, expand=False, fill=True, padding=0)
             self.screen_container.filter_vbox.reorder_child(
                 context_widget, 0)
             self.context_screen.widget.show()
@@ -129,8 +128,6 @@ class Screen(SignalEvent):
         self.search_value = attributes.get('search_value')
         self.fields_view_tree = {}
         self.order = self.default_order = attributes.get('order')
-        self.__date_format = self.context.get(
-            'date_format', rpc.CONTEXT.get('locale', {}).get('date', '%x'))
         self.view_to_load = []
         self._domain_parser = {}
         self.pre_validate = False
@@ -172,7 +169,7 @@ class Screen(SignalEvent):
             view_tree = self.fields_view_tree[view_id]
 
         fields = copy.deepcopy(view_tree['fields'])
-        for name, props in fields.iteritems():
+        for name, props in fields.items():
             if props['type'] not in ('selection', 'reference'):
                 continue
             if isinstance(props['selection'], (tuple, list)):
@@ -183,11 +180,26 @@ class Screen(SignalEvent):
             # Filter only fields in XML view
             xml_dom = xml.dom.minidom.parseString(view_tree['arch'])
             root_node, = xml_dom.childNodes
-            xml_fields = [node_attributes(node).get('name')
-                for node in root_node.childNodes
-                if node.nodeName == 'field']
-            fields = collections.OrderedDict(
-                (name, fields[name]) for name in xml_fields)
+            ofields = collections.OrderedDict()
+            for node in root_node.childNodes:
+                if node.nodeName != 'field':
+                    continue
+                attributes = node_attributes(node)
+                name = attributes['name']
+                # If a field is defined multiple times in the XML,
+                # take only the first definition
+                if name in ofields:
+                    continue
+                ofields[name] = fields[name]
+                for attr in ['string', 'factor']:
+                    if attributes.get(attr):
+                        ofields[name][attr] = attributes[attr]
+            fields = ofields
+
+        if 'active' in view_tree['fields']:
+            self.screen_container.but_active.show()
+        else:
+            self.screen_container.but_active.hide()
 
         # Add common fields
         for name, string, type_ in (
@@ -199,16 +211,14 @@ class Screen(SignalEvent):
                 ):
             if name not in fields:
                 fields[name] = {
-                    'string': string.decode('utf-8'),
+                    'string': string,
                     'name': name,
                     'type': type_,
                     }
                 if type_ == 'datetime':
                     fields[name]['format'] = '"%H:%M:%S"'
 
-        context = rpc.CONTEXT.copy()
-        context.update(self.context)
-        domain_parser = DomainParser(fields, context)
+        domain_parser = DomainParser(fields, self.context)
         self._domain_parser[view_id] = domain_parser
         return domain_parser
 
@@ -223,12 +233,12 @@ class Screen(SignalEvent):
                     props['selection'])
         except RPCException:
             selection = []
-        selection.sort(lambda x, y: cmp(x[1], y[1]))
+        selection.sort(key=itemgetter(1))
         return selection
 
     def search_prev(self, search_string):
         if self.limit:
-            self.offset -= self.limit
+            self.offset = max(self.offset - self.limit, 0)
         self.search_filter(search_string=search_string)
 
     def search_next(self, search_string):
@@ -246,7 +256,7 @@ class Screen(SignalEvent):
                 self.clear()
                 self.context_screen.display(set_cursor=True)
                 return False
-            context = self.context
+            context = self.group._context.copy()
             context.update(self.context_screen.get_on_change_value())
             self.new_group(context)
 
@@ -258,16 +268,24 @@ class Screen(SignalEvent):
         if tab_domain:
             domain = ['AND', domain, tab_domain]
 
-        try:
-            ids = RPCExecute('model', self.model_name, 'search', domain,
-                self.offset, self.limit, self.order, context=self.context)
-        except RPCException:
-            ids = []
+        context = self.context
+        if self.screen_container.but_active.get_active():
+            context['active_test'] = False
+        ids = []
+        while True:
+            try:
+                ids = RPCExecute('model', self.model_name, 'search', domain,
+                    self.offset, self.limit, self.order, context=context)
+            except RPCException:
+                break
+            if ids or self.offset <= 0:
+                break
+            self.offset = max(self.offset - self.limit, 0)
         if not only_ids:
             if self.limit is not None and len(ids) == self.limit:
                 try:
                     self.search_count = RPCExecute('model', self.model_name,
-                        'search_count', domain, context=self.context)
+                        'search_count', domain, context=context)
                 except RPCException:
                     self.search_count = 0
             else:
@@ -294,6 +312,7 @@ class Screen(SignalEvent):
                 domain = self.domain_parser.parse(search_string)
             else:
                 domain = self.search_value
+                self.search_value = None
             if set_text:
                 self.screen_container.set_text(
                     self.domain_parser.string(domain))
@@ -306,6 +325,11 @@ class Screen(SignalEvent):
         else:
             domain = self.domain
 
+        if self.screen_container.but_active.get_active():
+            if domain:
+                domain = [domain, ('active', '=', False)]
+            else:
+                domain = [('active', '=', False)]
         if self.current_view and self.current_view.view_type == 'calendar':
             if domain:
                 domain = ['AND', domain, self.current_view.current_domain()]
@@ -333,22 +357,24 @@ class Screen(SignalEvent):
 
     @property
     def context(self):
-        return self.group.context
-
-    @property
-    def date_format(self):
-        return self.__date_format
+        context = self.group.context
+        if self.context_screen:
+            context['context_model'] = self.context_screen.model_name
+        return context
 
     def __get_group(self):
         return self.__group
 
     def __set_group(self, group):
         fields = {}
+        fields_views = {}
         if self.group is not None:
             self.group.signal_unconnect(self)
-            for name, field in self.group.fields.iteritems():
+            for name, field in self.group.fields.items():
                 fields[name] = field.attrs
+                fields_views[name] = field.views
         self.tree_states_done.clear()
+        self.order = None
         self.__group = group
         self.parent = group.parent
         self.parent_name = group.parent_name
@@ -365,6 +391,8 @@ class Screen(SignalEvent):
             self._record_modified)
         self.__group.signal_connect(self, 'group-changed', self._group_changed)
         self.__group.add_fields(fields)
+        for name, views in fields_views.items():
+            self.__group.fields[name].views.update(views)
         self.__group.exclude_field = self.exclude_field
 
     group = property(__get_group, __set_group)
@@ -407,42 +435,24 @@ class Screen(SignalEvent):
                 # XXX offset?
                 pos = record.get_index_path()
         else:
-            pos = None
-        self.signal('record-message', (pos or 0, len(self.group) + self.offset,
+            pos = 0
+        self.signal('record-message', (pos, len(self.group) + self.offset,
             self.search_count, record and record.id))
-        attachment_count = 0
-        if record and record.attachment_count > 0:
-            attachment_count = record.attachment_count
-        self.signal('attachment-count', attachment_count)
-        unread_note = 0
-        if record and record.unread_note > 0:
-            unread_note = record.unread_note
-        self.signal('unread-note', unread_note)
-        # update attachment-count after 1 second
-        gobject.timeout_add(1000, self.update_attachment, record)
-        gobject.timeout_add(1000, self.update_note, record)
+        self.signal('resources', record.resources if record else None)
+        # update resources after 1 second
+        GLib.timeout_add(1000, self._update_resources, record)
         return True
 
     current_record = property(__get_current_record, __set_current_record)
 
-    def update_attachment(self, record):
-        if record != self.current_record:
-            return False
-        if record and self.signal_connected('attachment-count'):
-            attachment_count = record.get_attachment_count()
-            self.signal('attachment-count', attachment_count)
-        return False
-
-    def update_note(self, record):
-        if record != self.current_record:
-            return False
-        if record and self.signal_connected('unread-note'):
-            unread_note = record.get_unread_note()
-            self.signal('unread-note', unread_note)
+    def _update_resources(self, record):
+        if (record
+                and record == self.current_record
+                and self.signal_connected('resources')):
+            self.signal('resources', record.get_resources())
         return False
 
     def destroy(self):
-        self.screen_container.destroy()
         for view in self.views:
             view.destroy()
         del self.views[:]
@@ -457,7 +467,7 @@ class Screen(SignalEvent):
                 'model': self.model_name,
                 'id': self.current_record.id if self.current_record else None,
                 'ids': [r.id for r in self.selected_records],
-                }, context=self.context.copy(), warning=False)
+                }, context=self.group._context.copy(), warning=False)
         else:
             if not self.modified():
                 self.switch_view(view_type='form')
@@ -467,7 +477,9 @@ class Screen(SignalEvent):
     def number_of_views(self):
         return len(self.views) + len(self.view_to_load)
 
-    def switch_view(self, view_type=None):
+    def switch_view(self, view_type=None, view_id=None):
+        if view_id is not None:
+            view_id = int(view_id)
         if self.current_view:
             self.current_view.set_value()
             if (self.current_record and
@@ -480,22 +492,36 @@ class Screen(SignalEvent):
                 self.set_cursor()
                 self.current_view.display()
                 return
-        if not view_type or self.current_view.view_type != view_type:
-            for i in xrange(self.number_of_views):
-                if len(self.view_to_load):
-                    self.load_view_to_load()
-                    self.__current_view = len(self.views) - 1
-                else:
-                    self.__current_view = ((self.__current_view + 1)
-                            % len(self.views))
-                if not view_type:
-                    break
-                elif self.current_view.view_type == view_type:
-                    break
+
+        def found():
+            if not self.current_view:
+                return False
+            elif not view_type and view_id is None:
+                return False
+            elif view_id is not None:
+                return self.current_view.view_id == view_id
+            else:
+                return self.current_view.view_type == view_type
+        while not found():
+            if len(self.view_to_load):
+                self.load_view_to_load()
+                self.__current_view = len(self.views) - 1
+            elif (view_id is not None
+                    and view_id not in {v.view_id for v in self.views}):
+                self.add_view_id(view_id, view_type)
+                self.__current_view = len(self.views) - 1
+                break
+            else:
+                self.__current_view = ((self.__current_view + 1)
+                        % len(self.views))
+            if not view_type and view_id is None:
+                break
+            if view_type and not view_id and not len(self.view_to_load):
+                break
         self.screen_container.set(self.current_view.widget)
         self.display()
         # Postpone set of the cursor to ensure widgets are allocated
-        gobject.idle_add(self.set_cursor)
+        GLib.idle_add(self.set_cursor)
 
     def load_view_to_load(self):
         if len(self.view_to_load):
@@ -542,8 +568,10 @@ class Screen(SignalEvent):
                 fields[field]['loading'] = \
                     self.group.fields[field].attrs['loading']
         self.group.add_fields(fields)
-        view = View.parse(self, xml_dom, view.get('field_childs'))
-        view.view_id = view_id
+        for field in fields:
+            self.group.fields[field].views.add(view_id)
+        view = View.parse(
+            self, view_id, view['type'], xml_dom, view.get('field_childs'))
         self.views.append(view)
 
         return view
@@ -567,7 +595,7 @@ class Screen(SignalEvent):
         self.current_record = record
         self.display()
         # Postpone set of the cursor to ensure widgets are allocated
-        gobject.idle_add(self.set_cursor, True)
+        GLib.idle_add(self.set_cursor, True)
         return self.current_record
 
     def new_model_position(self):
@@ -581,11 +609,14 @@ class Screen(SignalEvent):
         if func_name:
             self.group.on_write.add(func_name)
 
-    def cancel_current(self):
+    def cancel_current(self, initial_value=None):
         if self.current_record:
             self.current_record.cancel()
             if self.current_record.id < 0:
-                self.remove(records=[self.current_record])
+                if initial_value is not None:
+                    self.current_record.reset(initial_value)
+                else:
+                    self.remove(records=[self.current_record])
 
     def save_current(self):
         if not self.current_record:
@@ -628,7 +659,7 @@ class Screen(SignalEvent):
         current_view = self.current_view
         if not current_view:
             return
-        elif current_view.view_type in ('tree', 'form'):
+        elif current_view.view_type in ('tree', 'form', 'list-form'):
             current_view.set_cursor(new=new, reset_view=reset_view)
 
     def get(self):
@@ -744,13 +775,9 @@ class Screen(SignalEvent):
         if parent is not None and parent < 0:
             return
         expanded_nodes, selected_nodes = [], []
-        timestamp = self.parent._timestamp if self.parent else None
         state = self.tree_states[parent][view.children_field]
         if state:
-            state_timestamp, expanded_nodes, selected_nodes = state
-            if (timestamp != state_timestamp
-                    and view.view_type != 'form'):
-                state = None
+            expanded_nodes, selected_nodes = state
         if state is None and CONFIG['client.save_tree_state']:
             json_domain = self.get_tree_domain(parent)
             try:
@@ -765,7 +792,7 @@ class Screen(SignalEvent):
                     _('Unable to get view tree state for %s')
                     % self.model_name)
             self.tree_states[parent][view.children_field] = (
-                timestamp, expanded_nodes, selected_nodes)
+                expanded_nodes, selected_nodes)
         if view.view_type == 'tree':
             view.expand_nodes(expanded_nodes)
             view.select_nodes(selected_nodes)
@@ -790,10 +817,9 @@ class Screen(SignalEvent):
 
     def save_tree_state(self, store=True):
         parent = self.parent.id if self.parent else None
-        timestamp = self.parent._timestamp if self.parent else None
         for view in self.views:
             if view.view_type == 'form':
-                for widgets in view.widgets.itervalues():
+                for widgets in view.widgets.values():
                     for widget in widgets:
                         if hasattr(widget, 'screen'):
                             widget.screen.save_tree_state(store)
@@ -803,12 +829,12 @@ class Screen(SignalEvent):
                         path = -self.current_record.group.index(
                             self.current_record)
                     self.tree_states[parent][view.children_field] = (
-                        timestamp, [], [[path]])
+                        [], [[path]])
             elif view.view_type == 'tree':
                 paths = view.get_expanded_paths()
                 selected_paths = view.get_selected_paths()
                 self.tree_states[parent][view.children_field] = (
-                    timestamp, paths, selected_paths)
+                    paths, selected_paths)
                 if (store
                         and int(view.attributes.get('tree_state', False))
                         and CONFIG['client.save_tree_state']):
@@ -821,6 +847,7 @@ class Screen(SignalEvent):
                             self.model_name, json_domain, view.children_field,
                             json_paths, json_selected_path,
                             process_exception=False)
+                        clear_cache('model.ir.ui.view_tree_state.get')
                     except Exception:
                         logger.warn(
                             _('Unable to set view tree state'), exc_info=True)
@@ -862,7 +889,13 @@ class Screen(SignalEvent):
             self.search_active(self.current_view.view_type
                 in ('tree', 'graph', 'calendar'))
             for view in self.views:
-                view.display()
+                # Always display tree view to update model
+                # because view can be used even if it is not shown
+                # like for save_tree_state
+                if (view == self.current_view
+                        or view.view_type == 'tree'
+                        or view.widget.get_parent()):
+                    view.display()
             self.current_view.widget.set_sensitive(
                 bool(self.group
                     or (self.current_view.view_type != 'form')
@@ -914,6 +947,11 @@ class Screen(SignalEvent):
                 record = next
                 break
             self.current_record = record
+        elif (view.view_type == 'list-form' and len(self.group)
+                and self.current_record in self.group):
+            idx = self.group.index(self.current_record)
+            if 0 <= idx < len(self.group) - 1:
+                self.current_record = self.group[idx + 1]
         elif view.view_type == 'calendar':
             record = self.current_record
             goocalendar = view.widgets.get('goocalendar')
@@ -1000,6 +1038,11 @@ class Screen(SignalEvent):
                             if prev_id >= 0:
                                 self.current_record = events[prev_id].record
                             break
+        elif (view.view_type == 'list-form' and len(self.group)
+                and self.current_record in self.group):
+            idx = self.group.index(self.current_record)
+            if 0 < idx <= len(self.group) - 1:
+                self.current_record = self.group[idx - 1]
         else:
             self.current_record = self.group[-1] if len(self.group) else None
         self.set_cursor(reset_view=False)
@@ -1010,7 +1053,7 @@ class Screen(SignalEvent):
             record = self.current_record
         domain_string = _('"%s" is not valid according to its domain')
         domain_parser = DomainParser(
-            {n: f.attrs for n, f in record.group.fields.iteritems()})
+            {n: f.attrs for n, f in record.group.fields.items()})
         fields = []
         for field, invalid in sorted(record.invalid_fields.items()):
             string = record.group.fields[field].attrs['string']
@@ -1098,7 +1141,7 @@ class Screen(SignalEvent):
 
     def _button_class(self, button):
         ids = [r.id for r in self.selected_records]
-        context = self.context.copy()
+        context = self.context
         context['_timestamp'] = {}
         for record in self.selected_records:
             context['_timestamp'].update(record.get_timestamp())
@@ -1108,7 +1151,7 @@ class Screen(SignalEvent):
         except RPCException:
             action = None
         self.reload(ids, written=True)
-        if isinstance(action, basestring):
+        if isinstance(action, str):
             self.client_action(action)
         elif action:
             Action.execute(action, {
@@ -1138,17 +1181,16 @@ class Screen(SignalEvent):
             self.display_prev()
         elif action == 'close':
             from tryton.gui import Main
-            Main.get_main().sig_win_close()
+            Main().sig_win_close()
         elif action.startswith('switch'):
-            _, view_type = action.split(None, 1)
-            self.switch_view(view_type=view_type)
+            self.switch_view(*action.split(None, 2)[1:])
         elif action == 'reload':
             if (self.current_view.view_type in ['tree', 'graph', 'calendar']
                     and not self.parent):
                 self.search_filter()
         elif action == 'reload menu':
             from tryton.gui import Main
-            RPCContextReload(Main.get_main().sig_win_menu)
+            RPCContextReload(Main().sig_win_menu)
         elif action == 'reload context':
             RPCContextReload()
 
@@ -1157,20 +1199,28 @@ class Screen(SignalEvent):
         if self.domain:
             query_string.append(('domain', json.dumps(
                         self.domain, cls=JSONEncoder, separators=(',', ':'))))
-        if self.context:
+        context = self.group._context  # Avoid rpc context
+        if context:
             query_string.append(('context', json.dumps(
-                        self.context, cls=JSONEncoder, separators=(',', ':'))))
+                        context, cls=JSONEncoder, separators=(',', ':'))))
         if self.context_screen:
             query_string.append(
                 ('context_model', self.context_screen.model_name))
         if name:
             query_string.append(
                 ('name', json.dumps(name, separators=(',', ':'))))
-        path = [rpc._DATABASE, 'model', self.model_name]
+        if self.screen_container.tab_domain:
+            query_string.append(('tab_domain', json.dumps(
+                        self.screen_container.tab_domain,
+                        cls=JSONEncoder, separators=(',', ':'))))
+        path = [CONFIG['login.db'], 'model', self.model_name]
         view_ids = [v.view_id for v in self.views] + self.view_ids
         if self.current_view.view_type != 'form':
-            search_string = self.screen_container.get_text()
-            search_value = self.domain_parser.parse(search_string)
+            if self.search_value:
+                search_value = self.search_value
+            else:
+                search_string = self.screen_container.get_text()
+                search_value = self.domain_parser.parse(search_string)
             if search_value:
                 query_string.append(('search_value', json.dumps(
                             search_value, cls=JSONEncoder,
@@ -1182,7 +1232,7 @@ class Screen(SignalEvent):
         if view_ids:
             query_string.append(('views', json.dumps(
                         view_ids, separators=(',', ':'))))
-        query_string = urllib.urlencode(query_string)
-        return urlparse.urlunparse(('tryton',
-                '%s:%s' % (rpc._HOST, rpc._PORT),
+        query_string = urllib.parse.urlencode(query_string)
+        return urllib.parse.urlunparse(('tryton',
+                CONFIG['login.host'],
                 '/'.join(path), query_string, '', ''))
