@@ -1,10 +1,8 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
+from gi.repository import Gtk
 
-from tryton.config import CONFIG
 from tryton.common.entry_position import reset_position
-from tryton.common.widget_style import set_widget_style
 from .widget import Widget
 
 
@@ -13,15 +11,15 @@ class TimeDelta(Widget):
     def __init__(self, view, attrs):
         super(TimeDelta, self).__init__(view, attrs)
 
-        self.widget = gtk.HBox()
-        self.entry = self.mnemonic_widget = gtk.Entry()
+        self.widget = Gtk.HBox()
+        self.entry = self.mnemonic_widget = Gtk.Entry()
         self.entry.set_alignment(1.0)
         self.entry.set_property('activates_default', True)
 
         self.entry.connect('activate', self.sig_activate)
         self.entry.connect('focus-out-event', lambda x, y: self._focus_out())
         self.entry.connect('key-press-event', self.send_modified)
-        self.widget.pack_start(self.entry)
+        self.widget.pack_start(self.entry, expand=True, fill=True, padding=0)
 
     @property
     def modified(self):
@@ -30,27 +28,22 @@ class TimeDelta(Widget):
             return self.field.get_client(self.record) != value
         return False
 
-    def set_value(self, record, field):
+    def set_value(self):
         value = self.entry.get_text()
-        return field.set_client(record, value)
+        return self.field.set_client(self.record, value)
 
     def get_value(self):
         return self.entry.get_text()
 
-    def display(self, record, field):
-        super(TimeDelta, self).display(record, field)
-        if not field:
+    def display(self):
+        super(TimeDelta, self).display()
+        if not self.field:
             value = ''
         else:
-            value = field.get_client(record)
+            value = self.field.get_client(self.record)
         self.entry.set_text(value)
         reset_position(self.entry)
 
     def _readonly_set(self, value):
         super(TimeDelta, self)._readonly_set(value)
         self.entry.set_editable(not value)
-        set_widget_style(self.entry, not value)
-        if value and CONFIG['client.fast_tabbing']:
-            self.widget.set_focus_chain([])
-        else:
-            self.widget.unset_focus_chain()
