@@ -1,12 +1,16 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
 import gettext
+
+from gi.repository import Gtk, Gdk
+
 import tryton.common as common
-from tryton.gui.window.view_form.screen import Screen
+from tryton.common.underline import set_underline
 from tryton.config import GNUHEALTH_ICON
-from tryton.gui.window.win_form import WinForm
+from tryton.gui import Main
 from tryton.gui.window.nomodal import NoModal
+from tryton.gui.window.view_form.screen import Screen
+from tryton.gui.window.win_form import WinForm
 
 _ = gettext.gettext
 
@@ -14,8 +18,8 @@ _ = gettext.gettext
 class WinSearch(NoModal):
 
     def __init__(self, model, callback, sel_multi=True, context=None,
-            domain=None, view_ids=None, views_preload=None, new=True,
-            title=''):
+            domain=None, order=None, view_ids=None,
+            views_preload=None, new=True, title=''):
         NoModal.__init__(self)
         if view_ids is None:
             view_ids = []
@@ -23,42 +27,61 @@ class WinSearch(NoModal):
             views_preload = {}
         self.domain = domain or []
         self.context = context or {}
+        self.order = order
         self.view_ids = view_ids
         self.views_preload = views_preload
         self.sel_multi = sel_multi
         self.callback = callback
         self.title = title
 
-        self.win = gtk.Dialog(_('Search'), self.parent,
-            gtk.DIALOG_DESTROY_WITH_PARENT)
+        self.win = Gtk.Dialog(
+            title=_('Search'), transient_for=self.parent,
+            destroy_with_parent=True)
+        Main().add_window(self.win)
         self.win.set_icon(GNUHEALTH_ICON)
-        self.win.set_default_response(gtk.RESPONSE_APPLY)
+        self.win.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
+        self.win.set_default_response(Gtk.ResponseType.APPLY)
         self.win.connect('response', self.response)
 
-        self.accel_group = gtk.AccelGroup()
+        self.win.set_default_size(*self.default_size())
+
+        self.accel_group = Gtk.AccelGroup()
         self.win.add_accel_group(self.accel_group)
 
-        self.but_cancel = self.win.add_button(gtk.STOCK_CANCEL,
-            gtk.RESPONSE_CANCEL)
-        self.but_find = self.win.add_button(gtk.STOCK_FIND, gtk.RESPONSE_APPLY)
+        self.but_cancel = self.win.add_button(
+            set_underline(_("Cancel")), Gtk.ResponseType.CANCEL)
+        self.but_cancel.set_image(common.IconFactory.get_image(
+                'tryton-cancel', Gtk.IconSize.BUTTON))
+        self.but_cancel.set_always_show_image(True)
+        self.but_find = self.win.add_button(
+            set_underline(_("Search")), Gtk.ResponseType.APPLY)
+        self.but_find.set_image(common.IconFactory.get_image(
+                'tryton-search', Gtk.IconSize.BUTTON))
+        self.but_find.set_always_show_image(True)
         if new and common.MODELACCESS[model]['create']:
-            self.but_new = self.win.add_button(gtk.STOCK_NEW,
-                gtk.RESPONSE_ACCEPT)
+            self.but_new = self.win.add_button(
+                set_underline(_("New")), Gtk.ResponseType.ACCEPT)
+            self.but_new.set_image(common.IconFactory.get_image(
+                    'tryton-create', Gtk.IconSize.BUTTON))
+            self.but_new.set_always_show_image(True)
             self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
 
-        self.but_ok = self.win.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
-        self.but_ok.add_accelerator('clicked', self.accel_group,
-                gtk.keysyms.Return, gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+        self.but_ok = self.win.add_button(
+            set_underline(_("OK")), Gtk.ResponseType.OK)
+        self.but_ok.set_image(common.IconFactory.get_image(
+                'tryton-ok', Gtk.IconSize.BUTTON))
+        self.but_ok.set_always_show_image(True)
+        self.but_ok.add_accelerator(
+            'clicked', self.accel_group, Gdk.KEY_Return,
+            Gdk.ModifierType.CONTROL_MASK, Gtk.AccelFlags.VISIBLE)
 
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.show()
-        self.win.vbox.pack_start(hbox, expand=False, fill=True)
-        self.win.vbox.pack_start(gtk.HSeparator(), expand=False, fill=True)
-        scrollwindow = gtk.ScrolledWindow()
-        scrollwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.win.vbox.pack_start(scrollwindow, expand=True, fill=True)
+        self.win.vbox.pack_start(hbox, expand=False, fill=True, padding=0)
+        self.win.vbox.pack_start(
+            Gtk.HSeparator(), expand=False, fill=True, padding=0)
 
-        self.screen = Screen(model, domain=domain, mode=['tree'],
+        self.screen = Screen(model, domain=domain, mode=['tree'], order=order,
             context=context, view_ids=view_ids, views_preload=views_preload,
             row_activate=self.sig_activate, readonly=True)
         self.view = self.screen.current_view
@@ -68,29 +91,20 @@ class WinSearch(NoModal):
         self.win.set_title(_('Search %s') % self.title)
 
         if not sel_multi:
-            sel.set_mode(gtk.SELECTION_SINGLE)
+            sel.set_mode(Gtk.SelectionMode.SINGLE)
         else:
-            sel.set_mode(gtk.SELECTION_MULTIPLE)
-        viewport = gtk.Viewport()
-        viewport.set_shadow_type(gtk.SHADOW_NONE)
-        viewport.add(self.screen.widget)
+            sel.set_mode(Gtk.SelectionMode.MULTIPLE)
+        self.win.vbox.pack_start(
+            self.screen.widget, expand=True, fill=True, padding=0)
         self.screen.widget.show()
-        viewport.show()
-        scrollwindow.add(viewport)
-        scrollwindow.show()
 
         self.model_name = model
 
-        self.win.set_default_size(700, 500)
-
         self.register()
-        sensible_allocation = self.sensible_widget.get_allocation()
-        self.win.set_default_size(int(sensible_allocation.width * 0.9),
-            int(sensible_allocation.height * 0.9))
 
     def sig_activate(self, *args):
-        self.view.treeview.emit_stop_by_name('row_activated')
-        self.win.response(gtk.RESPONSE_OK)
+        self.view.treeview.stop_emission_by_name('row_activated')
+        self.win.response(Gtk.ResponseType.OK)
         return True
 
     def destroy(self):
@@ -100,23 +114,22 @@ class WinSearch(NoModal):
 
     def show(self):
         self.win.show()
-        common.center_window(self.win, self.parent, self.sensible_widget)
 
     def hide(self):
         self.win.hide()
 
     def response(self, win, response_id):
         res = None
-        if response_id == gtk.RESPONSE_OK:
+        if response_id == Gtk.ResponseType.OK:
             res = [r.id for r in self.screen.selected_records]
-        elif response_id == gtk.RESPONSE_APPLY:
+        elif response_id == Gtk.ResponseType.APPLY:
             self.screen.search_filter(self.screen.screen_container.get_text())
             return
-        elif response_id == gtk.RESPONSE_ACCEPT:
+        elif response_id == Gtk.ResponseType.ACCEPT:
             # Remove first tree view as mode if form only
             view_ids = self.view_ids[1:]
             screen = Screen(self.model_name, domain=self.domain,
-                context=self.context, mode=['form'],
+                context=self.context, order=self.order, mode=['form'],
                 view_ids=view_ids, views_preload=self.views_preload)
 
             def callback(result):
